;;;   TRIMDIM.LSP
;;;   Copyright (C) 10-1995 by K & N Computer Group.
;;;      
;;;   DESCRIPTION
;;;     
;;;    This is procedure to trim dimension at line edg selected by user. 
;;;     
;;; ===================== load-time error checking ============================
;;;

  (defun ai_abort (app msg)
     (defun *error* (s)
	(if old_error (setq *error* old_error))
	(princ)
     )
     (if msg
       (alert (strcat " Application error: "
		      app
		      " \n\n  "
		      msg
		      "  \n"
	      )
       )
     )
     (exit)
  )

;;; Check to see if AI_UTILS is loaded, If not, try to find it,
;;; and then try to load it.
;;;
;;; If it can't be found or it can't be loaded, then abort the
;;; loading of this file immediately, preserving the (autoload)
;;; stub function.

  (cond
     (  (and ai_dcl (listp ai_dcl)))          ; it's already loaded.

     (  (not (findfile "ai_utils.lsp"))                     ; find it
	(ai_abort "TRIMDIM"
		  (strcat "Can't locate file AI_UTILS.LSP."
			  "\nCheck support directory.")))

     (  (eq "failed" (load "ai_utils" "failed"))            ; load it
	(ai_abort "TRIMDIM" "Can't load file AI_UTILS.LSP"))
  )

  (if (not (ai_acadapp))               ; defined in AI_UTILS.LSP
      (ai_abort "TRIMDIM" nil)        ; a Nil <msg> supresses
  )                                    ; ai_abort's alert box dialog.

;;; ==================== end load-time operations ===========================
;;; Main Procedure

(defun trimdim (/ trimdim_osm trimdim_oce trimdim_oer 
		   trimdim_err trimdim_oreg)

  (setq trimdim_osm (getvar "osmode")
	trimdim_oce (getvar "cmdecho")
	trimdim_oreg (getvar "regenmode")
  )

  ;;
  ;; Internal error handrer defined locally
  ;;

  (defun trimdim_err (s)               ;If an error (such as CTRL-C) occurs
				     ; while this command is active...
    (if (/= s "Function cancelled")
      (if (= s "quit / exit abort")
	(princ)
	(princ (strcat "\nError: " s))
      )
    )
    (command "_.UNDO" "_END")
    (ai_undo_off)
    (if trimdim_oer                        ; If an old error routine exists
      (setq *error* trimdim_oer)           ; then, reset it 
    )
    (if trimdim_osm (setvar "osmode" trimdim_osm))
    (if trimdim_oreg(setvar "regenmode" trimdim_oreg))
    (if trimdim_oce (setvar "cmdecho" trimdim_oce))      
    (princ)
  )
  
;;; Set new error handrer
    (if (not *DEBUG*)
      (if *error*
	 (setq trimdim_oer *error* *error* trimdim_err)
	 (setq *error* trimdim_err)
      )
    )

  (setvar "cmdecho"  0)
  (setvar "regenmode" 0)
  (setvar "osmode" 0)  
  (ai_undo_on)                       ; Turn on UNDO
  (command "_.UNDO" "_GROUP")
  (graphscr)
  (tdim_main)
  (command "_.UNDO" "_END")
  (ai_undo_off)                      ; Return UNDO to initial state
  (if trimdim_osm (setvar "osmode" trimdim_osm))
  (if trimdim_oce (setvar "cmdecho" trimdim_oce))      
  (if trimdim_oreg (setvar "regenmode" trimdim_oreg))
  (princ)
)
;;; ==================== end Main Procedure  ===========================
;;;
;;; Main functions.
;;;
;;; tdim_main  
;;;
(defun tdim_main ( / dimset setlen index item pt1 pt2 pt3 pt4
		     edgs_line star_pt end_pt exo_var style_dim dwg_scale)
   (setq index 0)
   (prompt "\nSelect dimension by Crossing : ")
   (setq dimset 
         (ssget '((0 . "DIMENSION")))
   )
   (setq setlen (sslength dimset))
   (initget 1)
   (setq edgs_line (entget (car (entsel "Select edgs for trim ..."))))
   (if (/= (cdr (assoc 0 edgs_line)) "LINE")
     (progn
       (princ "\nInvalid edgs. Trim edgs must be a line !")
       (princ)
       (exit)
     )
     (progn
       (setq star_pt (cdr (assoc 10 edgs_line)))
       (setq end_pt  (cdr (assoc 11 edgs_line)))
     )
   )
   (repeat setlen
     (setq item (entget (ssname dimset index)))
     (setq pt1    (cdr (assoc 13 item))
           pt2    (cdr (assoc 14 item))
           pt4    (cdr (assoc 10 item))
     )
     (if (= (car pt2) (car pt4))                 ;; horizional dimension
	(setq pt3 (list (car pt1) (cadr pt4) 0.0))
	(setq pt3 (list (car pt4) (cadr pt1) 0.0)) ;; else vertical dimension
     )
     (setq pt1 (inters pt1 pt3 star_pt end_pt nil)
           pt2 (inters pt2 pt4 star_pt end_pt nil)
     )
     (if pt1
     	(setq item 
       	   (subst (cons 13 pt1)
	          (assoc 13 item)
	      	  item
           )
        )
     )
     (if pt2	
        (setq item 
           (subst (cons 14 pt2)
  	          (assoc 14 item)
	          item
           )
        )
     )
     (entmod item)
     (setq index (+ 1 index))
   )
)

;;;
;;;
(defun c:td() 
   (trimdim)
)
(princ "\nType TD to star procedure trim dimension ...")
(princ)
